﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;


namespace DemoGridding
{
    public partial class Form1 : Form
    {
        int DISTANCE_RC_OK;
        int DISTANCE_RC_ERROR;
        int DISTANCE_RC_NO_MEMORY;
        int DISTANCE_RC_EXIST;

        // Upper limit for count of data points
        int GlMaxCount;

        public int GlnMethod;

        // Count of data points
        int GlnCount;


        // X, Y, Z values of data points
        double[] GlardX = new double[1000000];
        double[] GlardY = new double[1000000];
        double[] GlardZ = new double[1000000];

        // Values of grid
        public double[] GlardV = new double[1000000];

        // Point types of grid
        public int[] GlarnT = new int[1000000];

        // Count of rows and columns of grid
        public int GlnRowCount;
        public int GlnColumnCount;

        // The grid coordinates
        public double[] GldColumnCoord = new double[1500];
        public double[] GldRowCoord = new double[1500];

        public int lObjectHandle1;
        public int lObjectHandle2;

        int GlnPoints;
        public double[] GldLineCoordX = new double[10000];
        public double[] GldLineCoordY = new double[10000];

        // Values of contours
        public double[] GdaContourValues = new double[100];

        int GlnDrawType;

        public Form1()
        {
            InitializeComponent();
        }

        public int ContourColor(Color c)
        {
            return (c.B << 16) | (c.G << 8) | c.R;
        }

        private void Init()
        {
            InitGridding();

            // Return codes for CONTOUR
            int CONTOUR_RC_OK = 0;
            int CONTOUR_RC_ERROR = -1;
//            int CONTOUR_RC_NO_MEMORY = -2;
            int CONTOUR_RC_EXIST = 1;
//            int CONTOUR_RC_NO_EXIST = 2;

            try
            {
                // Create Contour object
                int nRC = Contour.ContourCreate();
                if (nRC != CONTOUR_RC_OK)
                {
                    if (nRC == CONTOUR_RC_EXIST)
                        System.Windows.Forms.MessageBox.Show("Object Contour exist");
                    if (nRC == CONTOUR_RC_ERROR)
                    {
                        System.Windows.Forms.MessageBox.Show("Can't create object Contour");
                        return;
                    }
                }
                return;
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function ContourCreate is missing");
                return;
            }
        }


        private void InitGridding()
        {
            int nRC;

            // Return codes for DISTANCE
            DISTANCE_RC_OK = 0;
            DISTANCE_RC_ERROR = -1;
            DISTANCE_RC_NO_MEMORY = -2;
            DISTANCE_RC_EXIST = 1;

            //   Upper limit for count of data points
            GlMaxCount = 1000000;
            //   Count of data points
            GlnCount = 0;

            //   Count of rows and columns of grid

            GlnRowCount = 50;
            GlnColumnCount = 50;

            int i;
            for (i = 0; i < GlnRowCount * GlnColumnCount; i++)
            {
                GlarnT[i] = 1;
            }

            GlnMethod = 0;

            try
            {
                // Create Triangulation object
                nRC = Distance.InvDstCreate();
                if (nRC != DISTANCE_RC_OK)
                {
                    if (nRC == DISTANCE_RC_EXIST)
                        System.Windows.Forms.MessageBox.Show("Object Distance exist");
                    if (nRC == DISTANCE_RC_ERROR)
                    {
                        System.Windows.Forms.MessageBox.Show("Can't create object Distance");
                        return;
                    }
                }
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function InvDstCreate is missing");
                return;
            }

            return;
        }


        private void button1_Click(object sender, System.EventArgs e)
        {
            DistanceInThread();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Init();
            OpenFile("d3.dat");
            Contour.ContourBitmapLoad("bg.png");
            Contour.ContourBitmapSetLocate(0, 0, 850, 650, 0.01, 0.9, 0.95, 0.02);
            DistanceInThread();
            GlnDrawType = 0;
        }

        private void Form1_Paint1(object sender, PaintEventArgs e)
        {
            if (GlnDrawType == 0)
                DrawWindowsContours();
                return;
        }

        private void DrawWindowsContours()
        {
            Graphics Mygraphics = pictureBox1.CreateGraphics();
            Mygraphics.Clear(Color.White);
            int longWidth = pictureBox1.Width;
            int longHeight = pictureBox1.Height;

            // Set canvas size in pixels
            int nRC = Contour.ContourCanvasSetSize(longWidth, longHeight);

            // Set HDC for canvas
            IntPtr hdc = Mygraphics.GetHdc();
            nRC = Contour.ContourCanvasSetHDC(hdc);

            DrawContours();

            Mygraphics.ReleaseHdc();

            Pen p = new Pen(Color.Red, (float)5.0);
            int x, y;
            for (int i = 0; i < GlnCount; i++)
            {
                x = Contour.ContourCanvasPixelX(GlardX[i]);
                y = Contour.ContourCanvasPixelY(GlardY[i]);
                Mygraphics.DrawEllipse(p, x, y, 5, 5);
            }

            return;
        }
        private void OpenFile(string sFile)
        {
            //   Open scattered data (X, Y, Z)
            int nCount;
            StreamReader fs;
            try
            {
                fs = File.OpenText(sFile);
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("File " + sFile + " is missing");
                return;
            }
            
            try
            {
                nCount = 0;
                string input = null;
                while ((input = fs.ReadLine()) != null)
                {
                    string [] split = input.Split(new Char [] {','});
                    if( nCount >= GlMaxCount )
                    {
                        System.Windows.Forms.MessageBox.Show("Too many points (" + Convert.ToString(GlMaxCount) + "is max)");
                        fs = null;                        //.close;
                        return;
                    }
                    GlardX[nCount] = Convert.ToDouble(split[0]);
                    GlardY[nCount] = Convert.ToDouble(split[1]);
                    GlardZ[nCount] = Convert.ToDouble(split[2]);
                    nCount++;
                }
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Data file record is invalid");

                fs = null;
                return;
            }
            //   Count of data points
            GlnCount = nCount;
            fs = null;
    
            return;            
        }

        private void DrawContours()
        {
            if (GlnRowCount > 50 || GlnColumnCount > 50)
               return;

            // Add the axis. The return code is axe index or -1 if fault
            int lBottom = Contour.ContourAxisAdd("bottom");
            int lTop = Contour.ContourAxisAdd("top");
            int lLeft = Contour.ContourAxisAdd("left");
            int lRight = Contour.ContourAxisAdd("right");

            // Change font height. The default is 18
            Contour.ContourAxisSetFontHeight(lBottom, 10);
            Contour.ContourAxisSetFontHeight(lTop, 10);
            Contour.ContourAxisSetFontHeight(lLeft, 10);
            Contour.ContourAxisSetFontHeight(lRight, 10);

            // Set the legends
            Contour.ContourAxisSetTitle(lBottom, "X, m");
            Contour.ContourAxisSetTitle(lLeft, "Y, m");
            Contour.ContourAxisSetTitle(lTop, "Top");

            // Set off the XY rate (1 - on, 0 - off)
            Contour.ContourCanvasSetXYRate(0);

            // Set view rectangle for canvas
            // The function set pixel-coordinate assigning

            int nRC = Contour.ContourCanvasSetLimits(GldColumnCoord[0], GldRowCoord[0], 
                GldColumnCoord[GlnColumnCount - 1], GldRowCoord[GlnRowCount - 1]);

            string sFont = "Times New Roman"; // Use TRUE TYPE fonts ONLY
            // Default font is Arial
            nRC = Contour.ContourCanvasSetFontFaceName(sFont);
            // Default font height is 20
            nRC = Contour.ContourCanvasSetFontHeight(15);

            Contour.ContourColorScaleSetFontFaceName(sFont);
            Contour.ContourColorScaleSetFontHeight(12);

            Contour.ContourColorScaleDraw();

            Contour.ContourAxisCalculate();

            Contour.ContourBitmapDraw();

            double dValue;
            int lVerticesCount;
            // Get count of color fill contours (polygons)
            int nCount = Contour.ContourPolygonsGetContoursCount();

            Contour.ContourCreateTransparentLayer();

            // Draw the color fill polygons

            // Get thee polygons in draw order
            int nColorIndex;
            for (int nI = 0; nI < nCount; nI++)
            {
                // Get vertices count and the value of contour
                GlnPoints = Contour.ContourPolygonsGetLineParameter(nI, out lVerticesCount, out dValue);
                // Get the color index of color bar for that polygon
                nColorIndex = Contour.ContourPolygonsGetLineColorIndex(nI);
                nRC = Contour.ContourPolygonsGetContourVertices(nI, out lVerticesCount, out GldLineCoordX[0], out GldLineCoordY[0]);

                // Set the color to draw that polygon
                Contour.ContourCanvasSetPolygonColorIndex(nColorIndex);
                Contour.ContourCanvasDrawPolygonContour(lVerticesCount, out GldLineCoordX[0], out GldLineCoordY[0]);
            }


            // Draw the contour (polylines) with the labels

            // Get count of contours (polylines)
            nCount = Contour.ContourPolygonsGetPolylinesCount();
            for (int nI = 0; nI < nCount; nI++)
            {
                // Get counter of vertices and the value of contour
                GlnPoints = Contour.ContourPolygonsGetPolylineParameter(nI, out lVerticesCount, out dValue);
                // Set the label value
                nRC = Contour.ContourCanvasSetLabelValue(dValue);

                nRC = Contour.ContourPolygonsGetPolylineVertices(nI, out lVerticesCount, out GldLineCoordX[0], out GldLineCoordY[0]);

                Contour.ContourCanvasDrawContour(lVerticesCount, out GldLineCoordX[0], out GldLineCoordY[0]);
            }
 
            // 100 is alpha canal value
            Contour.ContourDrawTransparentLayer(100);

            Contour.ContourAxisSimpleDraw();

            return;
        }


        private void CalculateMaxDistance(out double dD)
        {
            //   Calculate maximum distance between points
            double dXmin, dXmax, dYmin, dYmax;
    
            //   Calculate the minimum and maximum points coordinates
            CalculateMinMaxPointsCoordinates(out dXmin, out dYmin, out dXmax, out dYmax);
            dD = Math.Sqrt((dXmax - dXmin) * (dXmax - dXmin) + (dYmax - dYmin) * (dYmax - dYmin));
        }

        private void CalculateMinMaxPointsCoordinates(out double dXmin, out double dYmin, out double dXmax, out double dYmax)
        {
            //   Calculate the minimum and maximum points coordinates
            dXmin = GlardX[0];
            dXmax = GlardX[0];
            dYmin = GlardY[0];
            dYmax = GlardY[0];
            for (int nI = 1; nI <= GlnCount - 1; nI++)
            {
                if (dXmin > GlardX[nI])
                    dXmin = GlardX[nI];
                if (dXmax < GlardX[nI])
                    dXmax = GlardX[nI];
                if (dYmin > GlardY[nI])
                    dYmin = GlardY[nI];
                if (dYmax < GlardY[nI])
                    dYmax = GlardY[nI];
            }
        }

        private void Form1_Resize(object sender, EventArgs e)
        {

        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            CheckThread();
        }
        private void CheckThread()
        {
            if(GlnMethod == 1)
            {
                CheckDistancesThread();
                return;
            }
            if(GlnMethod == 2)
            {
                return;
            }
            return;
        }

        private void CheckDistancesThread()
        {
            int nCod = Distance.InvDstIsFinish();
            if( nCod == 1)
            {
                TimerText.Text = "Finish";
                timer1.Enabled = false;   // Set off Timer interval.
                FinishThread();
                return;
            }
            nCod = Distance.InvDstGetCurrentProgress();
            TimerText.Text = "Progress " + Convert.ToString(nCod);
            progressBar1.Value = nCod;
        }
        
        private void FinishDistanceThread()
        {
            int nRC;
            try
            {
                // Get values of grid nodes
                nRC = Distance.InvDstGetValues(out GlardV[0]);

                button6.Visible = true;
                buttonCancel.Visible = false;
                progressBar1.Visible = false;
                return;
            }    
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function InvDstGetValues is invalid");
                return;
            }   
        }

        private void FinishThread()
        {
            if(GlnMethod == 1)
            {
                FinishDistanceThread();
                return;
            }
            button6.Visible = true;
            buttonCancel.Visible = false;
            progressBar1.Visible = false;
            return;
        }
        private void CancelThread()
        {
            if (GlnMethod == 1)
            {
                Distance.InvDstCancel();
                return;
            }
        }
        private void CreateGrid()
        {
            //   Create the grid
    
            double dXmin, dXmax, dYmin, dYmax, dXStep, dYStep;
            int nJ;
            //   Calculate the minimum and maximum points coordinates
            CalculateMinMaxPointsCoordinates(out dXmin, out dYmin, out dXmax, out dYmax);

            double dXMargin = (dXmax - dXmin) * .05;
            double dYMargin = (dYmax - dYmin) * .05;

            dXmin -= dXMargin;
            dXmax += dXMargin;

            dYmin -= dYMargin;
            dYmax += dXMargin;

            //   Calculate the steps of grid
            dXStep = (dXmax - dXmin) / (GlnColumnCount - 1);
            dYStep = (dYmax - dYmin) / (GlnRowCount - 1);
    
            //   Calculate the coordinates of grid
            GldColumnCoord[0] = dXmin;
            for( nJ = 1; nJ <= GlnColumnCount - 1; nJ++ )
            {
                GldColumnCoord[nJ] = GldColumnCoord[nJ - 1] + dXStep;
            }
        
            GldRowCoord[0] = dYmin;
            for (nJ = 1; nJ <= GlnRowCount - 1; nJ++)
            {
                GldRowCoord[nJ] = GldRowCoord[nJ - 1] + dYStep;
            }

        }

        private void DistanceInThread()
        {
            int nRC;
            //Dim Start, Finish, TotalTime
            button6.Visible = false;
        
            //   Create the grid
            CreateGrid();
            try
            {
      
                // Set data points
                nRC = Distance.InvDstSetGrid(GlnColumnCount, out GldColumnCoord[0], GlnRowCount, out GldRowCoord[0]);
                if (nRC == DISTANCE_RC_ERROR || nRC == DISTANCE_RC_NO_MEMORY)
                {
                    System.Windows.Forms.MessageBox.Show("Object Distance can't create a grid");
                    return;
                }
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function InvDstSetGrid is missing");
                return;
            }       
            try
            {

                // Set data points
                nRC = Distance.InvDstSetData(GlnCount, out GlardX[0], out GlardY[0], out GlardZ[0]);
                if (nRC == DISTANCE_RC_ERROR || nRC == DISTANCE_RC_NO_MEMORY)
                {
                    System.Windows.Forms.MessageBox.Show("Object Distance can't set data");
                    return;
                }
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function InvDstSetData is missing");
                return;
            }       
            try
            {
   
                // Options
                
                // The calculation of a grid node will be done from the data points inside circle with reserch radius
                // The default search radius is maximum distance between data pointers
                double dRadius = Distance.InvDstGetSearchRadius();
                Distance.InvDstSetSearchRadius(dRadius);
                
                // The Power parameter controls how the weighting factors drop off as distance from a grid node increases
                // The default power is 2
                double dPower = Distance.InvDstGetPower();
                Distance.InvDstSetPower(dPower);
                
                // The calculation of a grid node will be done from lNearestPoints neighbour data points only
                // The default is count of all data points
                int lNearestPoints = Distance.InvDstGetNearestPoints();
                Distance.InvDstSetNearestPoints(10);
                
                // If distance from data point to grid node < dTolerance then value of node = value of this point
                // The default tolerance is 1/10 of cell length
                double dTolerance = Distance.InvDstGetTolerance();
                Distance.InvDstSetTolerance(dTolerance);

                // Run solver
                nRC = Distance.InvDstCalculateInThread();
                if (nRC == DISTANCE_RC_ERROR || nRC == DISTANCE_RC_NO_MEMORY)
                {
                    System.Windows.Forms.MessageBox.Show("Object Distance can't calculate");
                    return;
                }
                timer1.Enabled = true;
                timer1.Interval = 10;   //' Set Timer interval.
    
                // Progress bar change from 0 to 1000
                buttonCancel.Visible = true;
                progressBar1.Minimum = 0;
                progressBar1.Value = progressBar1.Minimum;
                progressBar1.Maximum = 1000;
                progressBar1.Visible = true;

                GlnMethod = 1;
                return;
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function InvDstCalculateInThread is invalid");
                return;
            }       
        }

        private void CreateContours()
        {
            double dMin, dMax, d;
            int nRC;

            // Return codes for CONTOUR
            // CONTOUR_RC_OK = 0;
            int CONTOUR_RC_ERROR = -1;
            int CONTOUR_RC_NO_MEMORY = -2;
            //CONTOUR_RC_EXIST = 1;
            //CONTOUR_RC_NO_EXIST = 2;


            if (GlnRowCount > 50 || GlnColumnCount > 50)
            {
                System.Windows.Forms.MessageBox.Show("Sorry. It is demo. Grid size must be no more 50x50");
                return;
            }

            try
            {

                nRC = Contour.ContourSetGridEx(GlnColumnCount, GlnRowCount, out GldColumnCoord[0], out GldRowCoord[0], out GlardV[0], out GlarnT[0]);

                if (nRC == CONTOUR_RC_ERROR || nRC == CONTOUR_RC_NO_MEMORY)
                {
                    System.Windows.Forms.MessageBox.Show("Object Contour can't create a grid");
                    return;
                }

                // Get the min and max value of grid field
                nRC = Contour.ContourGridGetMinMax(out dMin, out dMax);

                // Set contour values array 
                d = dMax - dMin;
                int nCount = 11;
                double ds;
                
                ds = d / (nCount - 1);
                
                int i;

                for (i = 0; i < nCount; i++)
                {
                    GdaContourValues[i] = dMin + i * ds;
                }

                Contour.ContourColorScaleCreate();
                Contour.ContourColorScaleSetColorCount(nCount);
                Contour.ContourColorScaleSetLimits(dMin, dMax);
                Contour.ContourColorScaleSetShowItemBoundaries(1);

                // Insert pink color in default color bar
                Color c = Color.FromArgb(255, 200, 200);
                Contour.ContourColorScaleSetColor(4, ContourColor(c));

                // Calculate contours

                nRC = Contour.ContourCalculateContours(nCount, out GdaContourValues[0]);
                return;
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function ContourCreate is missing");
                return;
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            CancelThread();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            DistanceInThread();
        }

        private void button5_Click(object sender, EventArgs e)
        {
        }

        private void button6_Click(object sender, EventArgs e)
        {
             CreateContours();
             DrawWindowsContours();
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            Contour.ContourDelete();
            Distance.InvDstDelete();
        }

        private void button7_Click(object sender, EventArgs e)
        {
            CreateBitmapFile();
        }

        private void CreateBitmapFile()
        {
            Graphics Mygraphics = pictureBox1.CreateGraphics();
            int longWidth = pictureBox1.Width;
            int longHeight = pictureBox1.Height;

            // Set canvas size in pixels
            int nRC = Contour.ContourCanvasSetSize(longWidth, longHeight);

            // Set HDC for canvas
            IntPtr hdc = Mygraphics.GetHdc();
            nRC = Contour.ContourCanvasSetHDC(hdc);

            Contour.ContourCanvasMemoryDraw();

            DrawContours();

            //            Contour.ContourCanvasWriteToPNG("test.png");
            // or (no and)
            Contour.ContourCanvasWriteToJPG("test.jpg");

            Mygraphics.ReleaseHdc();
        }
    }
}
